define([
        '../../Core/defaultValue'
    ],function(
        defaultValue) {
    'use strict';

    var feetPerMeter = 0.3048;  // exact value
    var metersToFeet = 1.0 / feetPerMeter;
    var squareMetersToSquareFeet = metersToFeet * metersToFeet;

    /**
     * Units of measure used for the measure widget.
     *
     * @exports MeasureUnits
     * @ionsdk
     */
    var MeasureUnits = {
        METERS : 'meters',
        FEET : 'feet'
    };

    /**
     * @private
     */
    MeasureUnits.numberToString = function(number, selectedLocale, fixed) {
        return numberToFormattedString(number, selectedLocale, fixed);
    };

    /**
     * @private
     */
    MeasureUnits.distanceToString = function(meters, selectedUnits, selectedLocale, fixed) {
        if (selectedUnits === MeasureUnits.METERS) {
            return numberToFormattedString(meters, selectedLocale, fixed) + 'm';
        }
        var feet = meters * metersToFeet;
        return numberToFormattedString(feet, selectedLocale, fixed) + 'ft';
    };

    /**
     * @private
     */
    MeasureUnits.areaToString = function(metersSquared, selectedUnits, selectedLocale, fixed) {
        if (selectedUnits === MeasureUnits.METERS) {
            return numberToFormattedString(metersSquared, selectedLocale, fixed) + 'm²';
        }
        var feet = metersSquared * squareMetersToSquareFeet;
        return numberToFormattedString(feet, selectedLocale, fixed) + 'sq ft';
    };

    function numberToFormattedString(number, selectedLocale, fixed) {
        fixed = defaultValue(fixed, 2);
        // If locale is undefined, the runtime's default locale is used.
        return number.toLocaleString(selectedLocale, {
            minimumFractionDigits: fixed,
            maximumFractionDigits: fixed
        });
    }

    return MeasureUnits;
});
