define([
        '../../Core/defaultValue',
        '../../Core/defined',
        '../../Core/Cartesian2',
        '../../Core/Cartesian3',
        '../../Core/Color',
        '../../Scene/HorizontalOrigin',
        '../../Scene/VerticalOrigin'
    ], function(
        defaultValue,
        defined,
        Cartesian2,
        Cartesian3,
        Color,
        HorizontalOrigin,
        VerticalOrigin) {
    'use strict';

    var defaultLabelPixelOffset = new Cartesian2(0, -9);

    /**
     * Contains options for configuring the style of the measurement widget primitives.
     *
     * @exports MeasurementSettings
     * @ionsdk
     */
    var MeasurementSettings = {};

    /**
     * Gets and sets the color used for the measurement primitives.
     * @type {Color}
     * @default Color.YELLOW
     */
    MeasurementSettings.color = Color.YELLOW;

    /**
     * Gets and sets the font used for the measurement labels.
     * @type {string}
     * @default '24px sans-serif'
     */
    MeasurementSettings.labelFont = '16px Lucida Console';

    /**
     * @private
     */
    MeasurementSettings.getPolylineOptions = function(options) {
        options = defaultValue(options, defaultValue.EMPTY_OBJECT);
        return {
            ellipsoid : options.ellipsoid,
            width : defaultValue(options.width, 3),
            color : MeasurementSettings.color,
            loop : options.loop,
            dashed : options.dashed,
            positions : options.positions
        };
    };

    /**
     * @private
     */
    MeasurementSettings.getPointOptions = function() {
        return {
            pixelSize : 10,
            color : MeasurementSettings.color,
            position : new Cartesian3(),
            disableDepthTestDistance : Number.POSITIVE_INFINITY, // for draw-over
            show : false
        };
    };

    /**
     * @private
     */
    MeasurementSettings.getLabelOptions = function(options) {
        options = defaultValue(options, defaultValue.EMPTY_OBJECT);
        return {
            show : false,
            font : MeasurementSettings.labelFont,
            scale : defaultValue(options.scale, 1.0),
            showBackground : true,
            horizontalOrigin : defaultValue(options.horizontalOrigin, HorizontalOrigin.CENTER),
            verticalOrigin : defaultValue(options.verticalOrigin, VerticalOrigin.BOTTOM),
            pixelOffset : defined(options.pixelOffset) ? options.pixelOffset : Cartesian2.clone(defaultLabelPixelOffset),
            disableDepthTestDistance : Number.POSITIVE_INFINITY, // for draw-over
            position : new Cartesian3()
        };
    };

    return MeasurementSettings;
});
